@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/child.c,v 1.8 2006/02/14 08:19:06 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/child.c,v 1.15 2006/05/11 03:00:07 y-asaba Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -82,6 +82,7 @@ void do_child(int unix_fd, int inet_fd)
 	static int connected;
 	int connections_count = 0;	/* used if child_max_connections > 0 */
 	int first_ready_for_query_received;		/* for master/slave mode */
+	int found;
 
 	pool_debug("I am %d", getpid());
 
@@ -120,6 +121,8 @@ void do_child(int unix_fd, int inet_fd)
 	timeout.tv_sec = pool_config.child_life_time;
 	timeout.tv_usec = 0;
 
+	init_prepared_list();
+
 	for (;;)
 	{
 		int connection_reuse = 1;
@@ -220,7 +223,37 @@ void do_child(int unix_fd, int inet_fd)
 		first_ready_for_query_received = 0;		/* for master/slave mode */
 
 		/* look for existing connection */
-		if ((backend = pool_get_cp(sp->user, sp->database, sp->major)) == NULL)
+		found = 0;
+		backend = pool_get_cp(sp->user, sp->database, sp->major);
+
+		if (backend != NULL)
+		{
+			found = 1;
+
+			/* existing connection associated with same user/database/major found.
+			 * however we should make sure that the startup packet contents identical.
+			 * OPTION data and others might be different.
+			 */
+			if (sp->len != backend->slots[0]->sp->len)
+			{
+				pool_debug("pool_process_query: connection exists but startup packet length is not identical");
+				found = 0;
+			}
+			else if(memcmp(sp->startup_packet, backend->slots[0]->sp->startup_packet, sp->len) != 0)
+			{
+				pool_debug("pool_process_query: connection exists but startup packet contents is not identical");
+				found = 0;
+			}
+
+			if (found == 0)
+			{
+				/* we need to discard existing connection since startup packet is different */
+				pool_discard_cp(sp->user, sp->database, sp->major);
+				backend = NULL;
+			}
+		}
+
+		if (backend == NULL)
 		{
 			/* create a new connection to backend */
 			connection_reuse = 0;
@@ -238,8 +271,16 @@ void do_child(int unix_fd, int inet_fd)
 
 		else
 		{
-			/* we don't need Startup Packet info anymore */
-			pool_free_startup_packet(sp);
+			/*
+			 * save startup packet info
+			 */
+			pool_free_startup_packet(backend->slots[0]->sp);
+			backend->slots[0]->sp = sp;
+
+			if (DUAL_MODE)
+			{
+				backend->slots[1]->sp = sp;
+			}
 
 			/* reuse existing connection to backend */
 
@@ -855,6 +896,9 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
 	return backend;
 }
 
+/*
+ * signal handler for SIGINT and SIGQUUT
+ */
 static RETSIGTYPE die(int sig)
 {
 	exit_request = 1;
@@ -884,6 +928,10 @@ static RETSIGTYPE die(int sig)
 	exit(0);
 }
 
+/*
+ * signal handler for SIGHUP
+ * close all idle connections
+ */
 static RETSIGTYPE close_idle_connection(int sig)
 {
 	int i;
@@ -989,7 +1037,7 @@ int health_check(void)
 	mysp.len = htonl(296);
 	mysp.sp.protoVersion = htonl(PROTO_MAJOR_V2 << 16);
 	strcpy(mysp.sp.database, "template1");
-	strcpy(mysp.sp.user, pool_config.health_check_user);
+ 	strncpy(mysp.sp.user, pool_config.health_check_user, sizeof(mysp.sp.user) - 1);
 	*mysp.sp.options = '\0';
 	*mysp.sp.unused = '\0';
 	*mysp.sp.tty = '\0';
