@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/pool_connection_pool.c,v 1.1.1.1 2005/02/14 14:52:21 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/pool_connection_pool.c,v 1.4 2006/05/11 02:15:55 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2005	Tatsuo Ishii
+ * Copyright (c) 2003-2006	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -194,7 +194,7 @@ void pool_connection_pool_timer(POOL_CONNECTION_POOL *backend)
 	POOL_CONNECTION_POOL *p = pool_connection_pool;
 	int i;
 
-	pool_debug("pool_connection_pool_timer: called");
+	pool_debug("pool_connection_pool_timer: set close time %d", time(NULL));
 
 	MASTER_CONNECTION(backend)->closetime = time(NULL);		/* set connection close time */
 
@@ -441,7 +441,7 @@ static POOL_CONNECTION_POOL *new_connection(POOL_CONNECTION_POOL *p)
 		create_cp(SECONDARY_CONNECTION(p), 1);
 
 		/* initialize Paramter Status save structure */
-		if (pool_init_params(&MASTER(p)->params))
+		if (pool_init_params(&SECONDARY(p)->params))
 		{
 			return NULL;
 		}
